<?php
namespace FiloBlu\Staging\Block\Adminhtml\Promo\Catalogrule\Edit\Tab\Products;


class Grid extends \Magento\Backend\Block\Widget\Grid\Extended
{
    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $moduleManager;

    /**
     * @var \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory]
     */
    protected $_setsFactory;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * @var \Magento\Catalog\Model\Product\Type
     */
    protected $_type;

    /**
     * @var \Magento\Catalog\Model\Product\Attribute\Source\Status
     */
    protected $_status;
    protected $_collectionFactory;

    /**
     * @var \Magento\Catalog\Model\Product\Visibility
     */
    protected $_visibility;

    /**
     * @var \Magento\Store\Model\WebsiteFactory
     */
    protected $_websiteFactory;

    protected $_request;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param \Magento\Store\Model\WebsiteFactory $websiteFactory
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory $setsFactory
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Catalog\Model\Product\Type $type
     * @param \Magento\Catalog\Model\Product\Attribute\Source\Status $status
     * @param \Magento\Catalog\Model\Product\Visibility $visibility
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param array $data
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        \Magento\Store\Model\WebsiteFactory $websiteFactory,
        \FiloBlu\Staging\Model\ResourceModel\Ruleproducts\CollectionFactory $productFactory,
        \Magento\Framework\Module\Manager $moduleManager,
        array $data = []
    ) {

        $this->_request = $context->getRequest();
        $collectionFactory =$productFactory;

        $this->_collectionFactory = $collectionFactory->create();
        $this->_websiteFactory = $websiteFactory;
        $this->moduleManager = $moduleManager;
        parent::__construct($context, $backendHelper, $data);
    }

    /**
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();

        $this->setId('flowGrid');
        $this->setDefaultSort('id');
        $this->setDefaultDir('DESC');
        $this->setSaveParametersInSession(true);
        $this->setUseAjax(true);

    }

    /**
     * @return Store
     */
    protected function _getId()
    {
        $Id = (int)$this->getRequest()->getParam('id', 0);
        return $Id;
    }

    /**
     * @return $this
     */
    protected function _prepareCollection()
    {
        try{

            $this->_collectionFactory->getSelect()
                ->join(
                array(
                    'cpe'=> "catalog_product_entity"),
                    'cpe.entity_id = main_table.product_id', array('cpe.sku')
            )
                ->join(
                    array(
                        'cg'=> "customer_group"),
                    'cg.customer_group_id = main_table.customer_group_id', array('group_concat(DISTINCT cg.customer_group_code ORDER BY cg.customer_group_code) as customer_groups')
                )
                ->join(
                    array(
                        'sw'=> "store_website"),
                    'sw.website_id = main_table.website_id', array('group_concat(DISTINCT sw.name ORDER BY sw.name) as websites')
                );

            $this->_collectionFactory->getSelect()->columns("FROM_UNIXTIME(from_time, '%Y-%m-%d %H:%i:%s') as start_date");
            $this->_collectionFactory->getSelect()->columns("FROM_UNIXTIME(to_time, '%Y-%m-%d %H:%i:%s') as to_date");
            $this->_collectionFactory->getSelect()->group("product_id");
            $collection =$this->_collectionFactory->load();
            $collection->addFieldToFilter('rule_id', $this->_getId());
            $this->setCollection($collection);

            parent::_prepareCollection();

            return $this;
        }
        catch(\Exception $e)
        {
            echo $e->getMessage();die;
        }
    }


    /**
     * @return $this
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareColumns()
    {

        $this->addColumn(
            'rule_id',
            [
                'header' => __('Rule Id'),
                'index' => 'rule_id',
                'class' => 'id'
            ]
        );
        $this->addColumn(
            'sku',
            [
                'header' => __('Sku'),
                'index' => 'sku',
                'class' => 'text'
            ]
        );
        $this->addColumn(
            'start_date',
            [
                'header' => __('From'),
                'index' => 'start_date',
                'class' => 'date'
            ]
        );
        $this->addColumn(
            'to_date',
            [
                'header' => __('To'),
                'index' => 'to_date',
                'class' => 'date'
            ]
        );
        $this->addColumn(
            'action_operator',
            [
                'header' => __('Operator'),
                'index' => 'action_operator',
                'class' => 'text'
            ]
        );
        $this->addColumn(
            'action_amount',
            [
                'header' => __('Valore'),
                'index' => 'action_amount',
                'class' => 'text'
            ]
        );
        $this->addColumn(
            'customer_groups',
            [
                'header' => __('Customer Group'),
                'index' => 'customer_groups',
                'class' => 'text'
            ]
        );
        $this->addColumn(
            'websites',
            [
                'header' => __('Website'),
                'index' => 'websites',
                'class' => 'text'
            ]
        );

        return parent::_prepareColumns();
    }


    /**
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl('rulestaging/ruleproduct/catalogRuleGrid', array('_current'=>true,'id'=> (int)$this->getRequest()->getParam('id', 0)));
    }

}
